﻿/*
 * LowBeam.h
 *
 * Author: Погребняк Дмитрий, г. Самара, 2014
 *
 * Помещённый здесь код является свободным. Т.е. допускается его свободное использование для любых целей, 
 * включая коммерческие, при условии указания ссылки на автора (Погребняк Дмитрий, http://aterlux.ru/).
 */ 


#ifndef LOWBEAM_H_
#define LOWBEAM_H_

#define F_CPU 1200000UL

// выключатель зажигания. Включено - низкий уровень. Подтягивающий резистор должен быть включен
#define IGNITION_DDR DDRB
#define IGNITION_PORT PORTB
#define IGNITION_PIN PINB
#define IGNITION_PIN_NUM 4
#define IGNITION (1 << IGNITION_PIN_NUM)

// выключатель фар. Включено - высокий уровень
#define SWITCH_DDR DDRB
#define SWITCH_PORT PORTB
#define SWITCH_PIN PINB
#define SWITCH_PIN_NUM 3
#define SWITCH (1 << SWITCH_PIN_NUM)

#define PWM_DDR DDRB
#define PWM_PORT PORTB
#define PWM_PIN_NUM 1
#define PWM (1 << PWM_PIN_NUM)
#define PWM_OC OCR0B

// Состояния конечного автомата
#define STATE_IGNITION_OFF 0
#define STATE_LIGHT_OFF 1
#define STATE_SLOW_WARM 2
#define STATE_FAST_WARM 3
#define STATE_ON 4
#define STATE_HOLD 5

// Количество тиков таймера в одной секунде (2353)
#define ONE_SECOND ((F_CPU + 255) / 510)

/* Настройки медленного прогрева.
 * Медленный прогрев осуществляется при включении фар, если до этого зажигание отключалось.
 * Выполняется в 4 стадии:
 * 1) разогрев от 0 до SLOW_RAMP_VALUE (64 = 25%) со скоростью SLOW_RAMP_SPEED (за 3 секунды)
 *  (.._VALUE - значение регистра OCR0B, 255 = 100%. 
 *   .._SPEED - количество тиков таймера минус один, до увеличения значения регистра OCR0B на единицу)
 * 2) удержание на достигнутом уровне в течение SLOW_RAMP_HOLD тиков (2 секунды)
 * 3) разогрев до SLOW_WARM_VALUE (204 = 80%) со скоростью SLOW_WARM_SPEED (за 3 секунды)
 * 4) разогрев до 100% со скоростью SLOW_LAST_SPEED (за 4 секунды)
 * Весь начальный прогрев занимает 3 + 2 + 3 + 4 = 12 секунд
 */
#define SLOW_RAMP_VALUE 64
#define SLOW_RAMP_SPEED (ONE_SECOND * 3 / SLOW_RAMP_VALUE)
#define SLOW_RAMP_HOLD (ONE_SECOND * 2)

#define SLOW_WARM_VALUE 204 
#define SLOW_WARM_SPEED (ONE_SECOND * 3 / (SLOW_WARM_VALUE - SLOW_RAMP_VALUE))
#define SLOW_LAST_SPEED (ONE_SECOND * 4 / (256 - SLOW_WARM_VALUE))

/* Настройки быстрого прогрева
 * Быстрый прогрев осуществляется при включении фар, если на период их отключения 
 * зажигание оставалось включенным. Выполняется в 2 стадии
 * 1) разогрев от текущего значения до FAST_WARM_VALUE (204 = 80%) со скоростью FAST_WARM_SPEED 
 *    (за 0,5 секунды, если раогрев от нуля)
 * 2) разогрев до 100% со скоростью FAST_LAST_SPEED (за 1 секунду)
 * Быстрый прогрев занимает 0,5 + 1 = 1,5 секунды
 */
#define FAST_WARM_VALUE 204
#define FAST_WARM_SPEED (ONE_SECOND / 2 / FAST_WARM_VALUE)
#define FAST_LAST_SPEED (ONE_SECOND / (256 - FAST_WARM_VALUE))

/* Удержание накала выполняется, если выключатель света отключен, а зажигание всё ещё включено.
 * Служит для сохранения накала ближнего света во время моргания дальним.
 * После выключения ближего света, накал удерживается на уровне HOLD_VALUE (128 = 50%) 
 * в течение HOLD_COUNTDOWN тиков (0,5 секунды), 
 * после чего накал плавно падает до 0% со скоростью HOLD_FALL_SPEED (за 0,5 секунды).
 * 
 * Значение HOLD_VALUE не должно слишком большим, поскольку в этом режиме схема питается через диод
 * от ламп, пока силовой транзистор закрыт. 
 */
#define HOLD_VALUE 128
#define HOLD_COUNTDOWN (ONE_SECOND / 2)
#define HOLD_FALL_SPEED (ONE_SECOND / 2 / HOLD_VALUE)

#endif /* LOWBEAM_H_ */